-- based on ButtonHole By joev (joev@jarhedz.com)

--
-- ENV
--
BH_VERSION = 1.5;

DEBUG = 0;

BH_COUNTER_MAX = 50;

PiraButtonHole = {
   -- Attributes
   modRegistry = nil;  
   player = nil;
   variablesLoaded = false;
   configLoaded = false;
   currentModId = nil;
   showingTooltip = false;
   updateCounter = 0;
   -- Constants
   const = {};
   -- Utility methods
   util = {};
   -- Application methods
   application = {};
   -- UI Management methods
   ui = {};
   -- Event handlers
   events = {};
};

RegistryEntry = {  
   id = nil;
   name = nil;
   tooltip = nil;
   buttonFrame = nil;
   updateFunction = nil;
   nextEntry = nil;
   previousEntry = nil;
   origUpdateFunction = nil;
};

PredefinedMods = {};
function PredefinedMods:load()
   
   PredefinedMods.Atlas = { 
      id="ATLAS", 
      name="Atlas", 
      tooltip = BHSTRINGS.preDefModsTooltip.Atlas, 
      buttonFrame="AtlasButtonFrame", 
      updateFunction="AtlasButton_UpdatePosition"
   };
   
   PredefinedMods.ELL = { 
      id="ELL",
      name="EnhancedLootLink",
      tooltip = "LootLink Enhanced",
      buttonFrame="LootLinkMinimapButton",
      updateFunction="LootLink_Minimap_Position"
   };  
   
   PredefinedMods.CT_RA = { 
      id="CTRA",
      name="CT_RaidAssist",
      tooltip = BHSTRINGS.preDefModsTooltip.CT_RaidAssist,
      buttonFrame="CT_RASets_Button",
      updateFunction="CT_RASets_MoveButton"
   };
   
   PredefinedMods.CT_Mod = { 
      id="CTMOD",
      name="CT_Mod",
      tooltip = BHSTRINGS.preDefModsTooltip.CT_Mod,
      buttonFrame="CT_OptionButton",
      updateFunction="CT_OptionButton_Move"
   };
   
   PredefinedMods.Gatherer = { 
      id="Gatherer",
      name="Gatherer",
      tooltip = BHSTRINGS.preDefModsTooltip.Gatherer,
      buttonFrame="GathererUI_IconFrame",
      updateFunction="GathererUI_MoveButton"
   }; 
   
   PredefinedMods.ItemRack = { 
      id="ItemRack",
      name="ItemRack",
      tooltip = "ItemRack",
      buttonFrame="ItemRack_IconFrame",
      updateFunction="ItemRack_move_icon"
   }; 
   
   PredefinedMods.FishingBuddy = { 
      id="FishingBuddy",
      name="FishingBuddy",
      tooltip = "FishingBuddy",
      buttonFrame="FishingBuddyMinimapFrame",
      updateFunction="FishingBuddyMinimapButton_MoveButton"
   };

   PredefinedMods.SWStats = { 
      id="SWSTATS", 
      name="SW Stats", 
      tooltip = "SW Stats", 
      buttonFrame="SW_IconFrame",
      updateFunction="SW_UpdateIconPos"
   };

end

BH_INFO = BHSTRINGS.appName.." "..BH_VERSION.." "..BHSTRINGS.by.." "..BHSTRINGS.author.name;

ModRegistryClass = {
};

PiraButtonHole.const = {
   textColors = {
      RED     = "|cffff0000";
      GREEN   = "|cff00ff00";
      BLUE    = "|cff0000ff";
      MAGENTA = "|cffff00ff";
      YELLOW  = "|cffffff00";
      CYAN    = "|cff00ffff";
      WHITE   = "|cffffffff";
      END     = "|r";
   };
};

--
--
--

-- constructor
function RegistryEntry:new(obj)
   local o = obj or {};
   setmetatable(o, self);
   self.__index = self;
   return o;
end

-- constructor
function ModRegistryClass:new(obj)
   local o = obj or {};
   setmetatable(o, self);
   self.__index = self;
   return o;
end

function ModRegistryClass:GetHead()
   local ret = nil;
   for key,val in self do 
      if (type(val) == "table") then
	 if (val.id and (not val.previousEntry)) then
	    PiraButtonHole.util.Debug("got head: "..val.id);
	    ret = val.id;
	    break;
	 end
      end
   end
   return ret;
end

function ModRegistryClass:GetTail() 
   local ret = nil;
   for key,val in self do 
      if (type(val) == "table") then
	 if (val.id and (not val.nextEntry)) then
	    PiraButtonHole.util.Debug("got tail: "..val.id);
	    ret = val.id;
	    break;
	 end
      end
   end
   return ret;
end

function ModRegistryClass:AddRegEntry(val)
   if (val ~= nil) then
      local str = val.id.." - "..val.name;
      PiraButtonHole.util.Debug("addedEntry:"..str);
      local tail = self:GetTail();
      self[val.id] = val;
      if (tail) then
	 PiraButtonHole.util.Debug("addEntry Tail:"..tail);
	 self[tail].nextEntry = self[val.id];
	 self[val.id].previousEntry = self[tail];
      end
   end
end

function ModRegistryClass:InsertRegEntry(val,prev)
   self[val.id] = val;
   if (prev == nil) then
      -- Inserting to the head of the this
      local currHead = self:GetHead();
      currHead.previousEntry = self[val.id];
      self[val.id].nextEntry = currHead;
   else
      self[val.id].nextEntry = prev.nextEntry;
      prev.nextEntry = self[val.id];
   end
end

function  ModRegistryClass:RemoveRegEntry(id)
   local ret = nil;
   if (self[id] ~= nil) then
      local preve = self[id].previousEntry;
      local nexte = self[id].nextEntry;
      nexte.previousEntry = preve;
      preve.nextEntry = nexte;
      ret = RegistryEntry:new(self[id]);
      self[id] = nil;
   end
   return ret;
end

PiraButtonHole.util = {  
   -- Does table contain the given value? 
   TableContainsKey = function(t, val)
			 local ret = false;
			 if (t and val) then
			    for key, value in t do
			       if (key == val) then
				  ret = true;
				  break;
			       end
			    end
			 end
			 return ret;
		      end;
   
   -- Print Debug info
   Debug = function (s)
	      if (DEBUG == 1) then
		 DEFAULT_CHAT_FRAME:AddMessage(s, 1, 1, 0)
	      end
	   end;
   
   -- Print to chat
   Print = function (s)
	      DEFAULT_CHAT_FRAME:AddMessage(s)
	   end;
    
   PrintTableValues = function(t)
			 for key,value in t do 
			    PiraButtonHole.util.Print(value) 
			 end    
		      end;
   
   PrintTableKeys = function(t)
		       for key,value in t do 
			  PiraButtonHole.util.Print(key) 
        end    
     end;
};

PiraButtonHole.ui = {
   -- Swallow a given frame into the hole
   SwallowFrame = function(frameName)
		     local frame = getglobal(frameName);
		     if (frame ~= nil) then
			frame:ClearAllPoints();
			frame:SetPoint("CENTER",
				       "PiraButtonHoleFrameBody",
				       "CENTER",
				       0,0);
			frame:Lower();
			frame:Hide();
		     end
		  end;
   -- Show the given mod entry
   ShowMod = function(regEntry)
		if (PiraButtonHole.modRegistry[regEntry] ~= nil) then
		   local frame = getglobal(PiraButtonHole.modRegistry[regEntry].buttonFrame);
		   if (frame ~= nil) then
		      for key,val in PiraButtonHole.modRegistry do
			 if (type(val) == "table" and val.id) then
			    PiraButtonHole.ui.HideMod(val.id);
			 end
		      end
		      frame:Raise();
		      frame:Show();
		      PiraButtonHole.ui.ShowTooltip(regEntry);
		      PiraButtonHole.currentModId = regEntry;
		      PiraButtonHoleConfig.visibleMod = regEntry;
		      PiraButtonHole.updateCounter = 0;
		   end
		end
	     end;
   -- Hide the given mod entry    
   HideMod = function(regEntry)
		if (PiraButtonHole.modRegistry[regEntry] ~= nil) then
		   local frame = getglobal(PiraButtonHole.modRegistry[regEntry].buttonFrame);
		   if (frame ~= nil) then
		      if (PiraButtonHole.showingTooltip) then
			 PiraButtonHole.ui.HideTooltip();
		      end
		      frame:Lower();
		      frame:Hide();
		   end
		end
	     end;
   -- Show the tooltip
   ShowTooltip = function (regEntry)
		    if (PiraButtonHole.modRegistry[regEntry]) then 
		       if (PiraButtonHole.modRegistry[regEntry].tooltip) then
			  GameTooltip:SetOwner(PiraButtonHoleFrame, "ANCHOR_LEFT");
			  GameTooltip:AddLine(PiraButtonHole.modRegistry[regEntry].name);
			  GameTooltip:AddLine(PiraButtonHole.modRegistry[regEntry].tooltip,.8,.8,.8,1);
			  GameTooltip:Show();
			  getglobal(PiraButtonHole.modRegistry[regEntry].buttonFrame):Raise();
			  PiraButtonHole.showingTooltip = true;
		       end
		    end
		 end;
   -- Hide the tooltip
   HideTooltip = function()
		    if (PiraButtonHole.showingTooltip) then
		       PiraButtonHole.showingTooltip = false;
		       GameTooltip:Hide();
		    end
		 end;
   
   Lock = function()
	     --PiraButtonHoleFrameHeader:Hide();
	     PiraButtonHoleFrameHeader:Show();	      
	     PiraButtonHoleConfig.locked = true;
	  end;
   
   Unlock = function()
	       PiraButtonHoleFrameHeader:Show();
	       PiraButtonHoleConfig.locked = false;
	    end;
   
   -- Move up the list and show the next button
   ScrollUp = function()
		 PiraButtonHole.util.Debug("scroll up");
		 if (PiraButtonHole.currentModId ~= nil) then
		    PiraButtonHole.ui.HideMod(PiraButtonHole.currentModId)
		    if (PiraButtonHole.currentModId == PiraButtonHole.modRegistry:GetTail()) then
		       -- We're at the tail - go back to head 
		       PiraButtonHole.currentModId = PiraButtonHole.modRegistry:GetHead();
		    else
		       PiraButtonHole.currentModId = PiraButtonHole.modRegistry[PiraButtonHole.currentModId].nextEntry.id;       
		    end
		    PiraButtonHole.ui.ShowMod(PiraButtonHole.currentModId);
		 end
	      end;
   
   -- Move down the list and show the next button
   ScrollDown = function()
		   PiraButtonHole.util.Debug("scroll down");
		   if (PiraButtonHole.currentModId ~= nil) then
		      PiraButtonHole.ui.HideMod(PiraButtonHole.currentModId)
		      if (PiraButtonHole.currentModId == PiraButtonHole.modRegistry:GetHead()) then
			 -- We're at the head - go to tail 
			 PiraButtonHole.currentModId = PiraButtonHole.modRegistry:GetTail();
		      else
			 PiraButtonHole.currentModId = PiraButtonHole.modRegistry[PiraButtonHole.currentModId].previousEntry.id;       
		      end
		      PiraButtonHole.ui.ShowMod(PiraButtonHole.currentModId);
		   end
		   
		end;
};

PiraButtonHole.application = {
   -- Register mod with PiraButtonHole
   RegisterMod = 
      function(modEntry)
	 if (modEntry and modEntry.id) then
	    if (not PiraButtonHole.util.TableContainsKey(PiraButtonHole.modRegistry, modEntry.id)) then
	       -- Only add the mod if it actually exists! (Catch the PreDefineds)
	       if (getglobal(modEntry.buttonFrame)) then
			     local str = modEntry.id..modEntry.name;
		  PiraButtonHole.util.Debug("Entry = "..str);
		  PiraButtonHole.modRegistry:AddRegEntry(modEntry);
	       else
		  PiraButtonHole.util.Debug("frame missing: "..modEntry.buttonFrame);
	       end
	    end
	 end
      end;
   
   -- Show slash command usage
   ShowUsage = function ()
		  PiraButtonHole.util.Print(BH_INFO);
		  PiraButtonHole.util.PrintTableValues(BHSTRINGS.usage);
	       end;
   
    -- Show the application credits.
   ShowCredits = function()
		    PiraButtonHole.util.Print(BHSTRINGS.appName.." v"..BH_VERSION.." - "..BHSTRINGS.creditsInfo.."\n\n");
		    
		    for key,value in BHSTRINGS.credits do
		       PiraButtonHole.util.Print(PiraButtonHole.const.textColors.CYAN..value.task);
		       if (value.alias == "") then
			  PiraButtonHole.util.Print(PiraButtonHole.const.textColors.YELLOW..value.name.."\n\n");
		       else
			  PiraButtonHole.util.Print(PiraButtonHole.const.textColors.YELLOW..value.name..BHSTRINGS.aka..value.alias.."\n\n");
		       end
		    end
		 end;
   
   -- Setup config variables and initialise the application.
   ConfigInit = function()
		   if (not PiraButtonHole.configLoaded) then
		      if (not PiraButtonHoleConfig) then
			 PiraButtonHoleConfig = {};
			 PiraButtonHoleConfig.on = true;
			 PiraButtonHoleConfig.visibleMod = nil;            
			 PiraButtonHoleConfig.locked = false;
			 PiraButtonHoleConfig.version = tonumber(BH_VERSION);
		      else
			 if (not PiraButtonHoleConfig.version) then
			    PiraButtonHoleConfig.version = 1;
			 end
			 if (PiraButtonHoleConfig.version < tonumber(BH_VERSION)) then
			    PiraButtonHoleConfig.version = tonumber(BH_VERSION);
			 end
		      end
		      PiraButtonHole.configLoaded = true;
		      PiraButtonHole.util.Debug("Finished Config Loading");
		      
		      PiraButtonHole.application.LoadPredefinedMods();
		      
		      PiraButtonHole.application.SetOn(PiraButtonHoleConfig.on);
		      
		      --PiraButtonHole.util.Print(BH_INFO..BHSTRINGS.initialized);
		   end
		end;
   
   -- Toggle the application on/off  
   SetOn = function(val)
	      PiraButtonHoleConfig.on = val;
	      PiraButtonHole.application.Setup();
	   end;
   
   -- Setup the application
   Setup = function()    
	      if (PiraButtonHole.application.IsOn()) then
		 -- Swallow the Mods in the registry into the hole... We're managing them now.
		 for key,val in PiraButtonHole.modRegistry do
		    if (type(val) == "table" and val.id) then
		       -- Swallow the frame
		       PiraButtonHole.ui.SwallowFrame(val.buttonFrame);
		       -- Hide it...
		       getglobal(val.buttonFrame):Hide();
		       -- Disable mods "move minimap button" method
		       local origFunc = getglobal(val.updateFunction);
		       if (origFunc == nil) then
			  PiraButtonHole.util.Debug("origFunc is nil... what's up with that?");
		       end
		       PiraButtonHole.util.Debug("attempting override of: "..val.updateFunction);
		       PiraButtonHole.util.Debug("trapping orig...");
		       PiraButtonHole.modRegistry[val.id].origUpdateFunction = origFunc;
		       PiraButtonHole.util.Debug("overriding with setglobal...");
		       setglobal(val.updateFunction, function (...) return (nil) end);
		       PiraButtonHole.util.Debug("Orig func overriden");
		    end  
		 end
		 if ((PiraButtonHoleConfig.visibleMod ~= nil) and (PiraButtonHole.modRegistry[regEntry] ~= nil)) then
		    PiraButtonHole.ui.ShowMod(PiraButtonHoleConfig.visibleMod)
		 else 
		    PiraButtonHole.ui.ShowMod(PiraButtonHole.modRegistry:GetHead());
		 end
		 if (PiraButtonHoleConfig.locked) then
		    --PiraButtonHole.util.Print("Locked position");
		    PiraButtonHole.ui.Lock();
		 else
		    --PiraButtonHole.util.Print("Unlocked position");
		    PiraButtonHole.ui.Unlock();
		 end
		 UIDropDownMenu_Initialize(PiraButtonHoleDropdownMenu, PiraButtonHole.application.DropDownInitialize, "MENU");
		 UIDropDownMenu_SetAnchor(0, 0, PiraButtonHoleDropdownMenu, "TOPRIGHT", "PiraButtonHoleFrameHeader", "TOPLEFT")
		 
	      end
	   end;
   
   -- Is the application on?
   IsOn = function()
	     return PiraButtonHoleConfig.on;
	  end;
   
   IsLocked = function()
		 return PiraButtonHoleConfig.locked;
	      end;
   
   -- Handle slash commands
   SlashHandler = function(msg)
		     if msg == nil or msg == "" then
			msg = "help";
		     end
		     msg = strlower(msg);
		     local args = {n=0}
		     local function helper(word) table.insert(args, word) end
		     string.gsub(msg, "[_%w]+", helper);
        if args[1] == 'on'  then
	   PiraButtonHole.application.SetOn(true);
	   PiraButtonHole.util.Print(PiraButtonHole.const.textColors.CYAN..BHSTRINGS.appName..PiraButtonHole.const.textColors.YELLOW.." :: "..PiraButtonHole.const.textColors.WHITE..BHSTRINGS.isNow..PiraButtonHole.const.textColors.GREEN..BHSTRINGS.on);
        elseif args[1] == 'off' then
	   PiraButtonHole.application.SetOn(false);
	   PiraButtonHole.util.Print(PiraButtonHole.const.textColors.CYAN..BHSTRINGS.appName..PiraButtonHole.const.textColors.YELLOW.." :: "..PiraButtonHole.const.textColors.WHITE..BHSTRINGS.isNow..PiraButtonHole.const.textColors.RED..BHSTRINGS.off);
        elseif args[1] == 'lock' then
	   PiraButtonHole.ui.Lock();
        elseif args[1] == 'unlock' then
	   PiraButtonHole.ui.Unlock();
        elseif args[1] == 'credits' then
	   PiraButtonHole.application.ShowCredits();
        else
	   PiraButtonHole.application.ShowUsage();
        end
     end;
   
   LoadPredefinedMods = function()
			   
			   PiraButtonHole.util.Debug("loading predefs");
			   PredefinedMods:load();
			   
			   for key,val in PredefinedMods do
			      if (type(val) == "table" and val.id) then      
				 PiraButtonHole.util.Debug("adding: "..val.id);
				 PiraButtonHole.application.RegisterMod(val);
			      end
			   end      
			end;
   
   ToggleLock = function()
		   if (PiraButtonHoleConfig.locked) then
		      PiraButtonHole.ui.Unlock();
		   else
		      PiraButtonHole.ui.Lock();
		   end
		end;
   
   DropDownInitialize = function()
			   local info;
			   info = {};
			   info.text = BHSTRINGS.appName.." "..BH_VERSION;
			   info.notClickable = 1;
			   info.isTitle = 1;
			   UIDropDownMenu_AddButton(info);
			   
			   info = {};
			   info.disabled = 1;
			   UIDropDownMenu_AddButton(info);
			   
			   info = {};
			   info.text = BHSTRINGS.dropdown.lock;
			   info.func = PiraButtonHole.application.ToggleLock;
			   info.checked = PiraButtonHole.application.IsLocked();
			   UIDropDownMenu_AddButton(info);
			   
			   info = {};
			   info.text = BHSTRINGS.dropdown.options;
			   info.disabled = 1;
			   --info.func = AcceptBattlefieldPort;
			   --info.notCheckable = 1;
			   UIDropDownMenu_AddButton(info);
			end;
};

--
-- EVENTS
--
PiraButtonHole.events = {
   
   -- Load event handler.
   OnLoad = function ()
	       this:RegisterEvent("VARIABLES_LOADED");
	       this:RegisterEvent("PLAYER_ENTERING_WORLD");
	       
	       -- Slash Commands
	       SlashCmdList["BH"] = PiraButtonHole.application.SlashHandler;
	       SLASH_BH1 = "/buttonhole";
	       SLASH_BH2 = "/bh";
	       
	       PiraButtonHole.modRegistry = ModRegistryClass:new();
	       
	       
	    end;
   
   -- General event handler.
   OnEvent = function()
		if (event == "VARIABLES_LOADED") then
		   PiraButtonHole.variablesLoaded = true;
		   PiraButtonHole.util.Debug("Variables Loaded");
		   if (PiraButtonHole.player) then
		      PiraButtonHole.application.ConfigInit();
		   end
		   -- Add myAddOns support
		   if myAddOnsList then
		      myAddOnsList.PiraButtonHole = {name = "PiraButtonHole", description = BHSTRINGS.myAddonsDescription, version = BH_VERSION, category = MYADDONS_CATEGORY_OTHER, frame = "PiraButtonHoleFrame", optionsframe = ""};
		   end
		   elseif( event == "PLAYER_ENTERING_WORLD" and not PiraButtonHole.player) then
		   local name = UnitName("player");
		   if( name and name ~= UNKNOWNOBJECT ) then
		      PiraButtonHole.player = name;
		      PiraButtonHole.util.Debug("Unit Name: player = "..PiraButtonHole.player);
		      if (PiraButtonHole.variablesLoaded) then
			 PiraButtonHole.application.ConfigInit();
		      end
		   end
		end        
	     end;
   
   -- Mousewheel event handler
   OnMouseWheel = function(value)
		     if (PiraButtonHole.application.IsOn()) then
			if ( value > 0 ) then 
			   PiraButtonHole.ui.ScrollUp();
			elseif ( value < 0 ) then 
			   PiraButtonHole.ui.ScrollDown();
			end
		     end
		  end;
   
   OnRightClick = function()
		     if (PiraButtonHole.application.IsOn()) then
			PiraButtonHole.ui.ScrollDown();   
		     end
		  end;
   
   OnUpdate = function()
		 if (PiraButtonHole.updateCounter > BH_COUNTER_MAX) then
		    -- Time to do something.
		    PiraButtonHole.updateCounter = 0;
		    if (MouseIsOver(PiraButtonHoleFrame) and not GameTooltip:IsShown()) then
		       PiraButtonHole.ui.ShowTooltip(PiraButtonHole.currentModId);
		       for key,val in PiraButtonHole.modRegistry do
			  if (type(val) == "table" and val.id) then
			     --Correct some errors that can occur
			     if (getglobal(val.buttonFrame):IsVisible() and (val.id ~= PiraButtonHole.currentModId)) then
				-- You're not supposed to be here... back in the Hole!
				PiraButtonHole.ui.HideMod(val.id);
			     end
			     if (not getglobal(PiraButtonHole.modRegistry[PiraButtonHole.currentModId].buttonFrame):IsVisible()) then
				-- It rubs the lotion on it's skin or else it gets the hose again...
				PiraButtonHole.ui.ShowMod(PiraButtonHole.currentModId);
			     end
			  end
		       end
		    else
		       if (not MouseIsOver(PiraButtonHoleFrame) and PiraButtonHole.showingTooltip) then
			  PiraButtonHole.ui.HideTooltip();
		       end
		    end
		 end
		 PiraButtonHole.updateCounter = PiraButtonHole.updateCounter + 1;
	      end;
   
   header = {
      OnEnter = function()
		   GameTooltip:SetOwner(PiraButtonHoleFrameHeader, "ANCHOR_LEFT");
		   GameTooltip:SetText(BH_INFO, 255/255, 209/255, 0/255);
		   GameTooltip:AddLine(BHSTRINGS.header.tooltip, 1.00, 1.00, 1.00);  
		   GameTooltip:Show();
		end;
      
      OnLeave = function()
		   GameTooltip:Hide();
		end;
   };
};
