-- **************************************************************************
-- * TitanHearthStone.lua
-- * This addon will add heartstone time and casting to your titan bar.
-- * This is my first .lua, so there may be mistakes in here. 
-- * I looked at the following mods for help:
-- * 1) ShardTracker - Awesome mod for warlocks
-- * 2) All the Capnbry stuff (www.capnbry.net)  Many "must have" mods.
-- * 3) Titan addon for "Skills". I used this for the basic framework to get me going.
-- *
-- * Note: My wife made me change some of the spelling of HearthStone to Hearthstone, so it might
-- * be varied in places in the code. If nothing else, I do know when to follow directions! ;)
-- *
-- * Additonal addons may appear from http://www.programming.org/wowmods.htm
-- *
-- * Version v.4
-- * 1) Local-scope fix
-- * 2) Updated german / french versions.
-- *
-- * Version v.3
-- * 1) Added Astral Recall Spell, which displays the spell cooldown timer, and upon
-- *    click, will attempt to cast that spell first.
-- * 2) German version, thanks to Patrick L.
-- * 3) Increased polling frequency to 5 seconds
-- * 
-- * Version v.2
-- * 1) Added french translation, thanks to Guillop.
-- * 
-- * Version History(v.11)
-- * 1) Changed the lookup to use GetBindLocation(), I wish I knew about that function earlier
-- * 2) Added hint text
-- * 3) TITAN_NIL instead of just nil for user variables.
-- **************************************************************************

-- ************************************* Const / defines ************************************* --
TITAN_HearthStone_FREQUENCY 			= 5.0;		-- update frequency in seconds, too keep us from checking too often. --
TITAN_HearthStone_NotFound				= 50000;	-- my own code for NOT FOUND --
TITAN_Cached_Cooldown					= 50000;	-- my own code for NOT FOUND --
TITAN_HearthStone_Hours					= 3600;		-- Seconds in the hour --
TITAN_HearthStone_Seconds				= 60;		-- minutes in an hour -- 
TITAN_NotFoundBag						= 1000;		-- Code for not found bag --
TITAN_NotFoundSlot						= 1000;		-- Code for not found slot --
TITAN_HearthStone_COLOR_GOLD			= "|cFFFFFF22"; -- color for location --
TITAN_CachedHome						= nil;		-- our home --
TITAN_Hearth_AstralRecallMana			= 150;		-- mana usage for this spell --

-- ************************************* Event functions ************************************* --
-- ******************************************************************************************* --
-- Name: TitanPanelHearthStoneButton_OnLoad
-- Desc: This function gets the ball rolling, called at the start
-- ******************************************************************************************* --
function TitanPanelHearthStoneButton_OnLoad()
	
	-- Called on load, this sets the ball rolling.
	this.registry = { 
		id 					= TITAN_HearthStone_ID,
		menuText 			= TITAN_HearthStone_MENU_TEXT, 
		buttonTextFunction 	= "TitanPanelHearthStoneButton_GetButtonText", 
		tooltipTitle 		= TITAN_HearthStone_TOOLTIP,
		tooltipTextFunction = "TitanPanelHearthStoneButton_GetTooltipText",
		icon 				= TITAN_HearthStone_ICON,
		frequency 			= TITAN_HearthStone_FREQUENCY, 					
		iconWidth 			= 16,
		savedVariables 		= {
								ShowIcon 			= 1,
								ShowLabelText 		= 1,
								ShowHearthLocation 	= TITAN_NIL,
								ActivateOnClick 	= TITAN_NIL,
							  }
	};

end

-- ******************************************************************************************* --
-- Name: TitanPanelHearthStoneButton_OnEvent
-- Desc: In truth, I don't know what this does.. I assume it maps all events to the events
-- 		 we want to listen to.
-- ******************************************************************************************* --
function TitanPanelHearthStoneButton_OnEvent()
	TitanPanelButton_UpdateButton(TITAN_HearthStone_ID);	
	TitanPanelButton_UpdateTooltip();
end

-- ******************************************************************************************* --
-- Name: TitanPanelHearthStoneButton_OnClick
-- Desc: Called when we click on our item. We do different things depending on if you do a 
-- 	     Right click, or a left click.
-- ******************************************************************************************* --
function TitanPanelHearthStoneButton_OnClick(button)

	-- On Right click, we drop down our menu
	if (button == "RightButton") then
		-- It was a right button click.. what to do....
		if (GameTooltip:IsVisible()) then
			GameTooltip:Hide();
		end
		
			-- Going to make our dropdown
			DropDownList1:Hide();
        	local x, y = GetCursorPosition(UIParent);
        	if ( position == TITAN_PANEL_PLACE_TOP ) then
            	   	TitanPanelHearthStone_Button_Menu.point = "TOPLEFT";
         	      	TitanPanelHearthStone_Button_Menu.relativePoint = "BOTTOMLEFT";
        	else
          	     	TitanPanelHearthStone_Button_Menu.point = "BOTTOMLEFT";
         	      	TitanPanelHearthStone_Button_Menu.relativePoint = "TOPLEFT";
        	end
        	ToggleDropDownMenu(1, nil, TitanPanelHearthStone_Button_Menu, "TitanPanelBarButton", x - 40, 0);
        	
    -- ELSE, we see if its a left button click.
	elseif (button == "LeftButton") then

		-- *********************************************** CAUTION ***********************************************
		-- Don't ask me why, but this is backwards from what it should be.. I have no idea why this works flipped.	
		-- I must be missing something here.. but it "works".. its just backwards logic. Next version, hopefully
		-- with some community help, I'll have this resolved. I mentioned before, this is day 1 with lua, so
		-- bound to be some mistakes in here.	
		-- *******************************************************************************************************
		if (not TitanGetVar(TITAN_HearthStone_ID, "ActivateOnClick")) then
		
			-- First, try the spell.
			local theSpellCooldown, theSpellID = TITAN_HearthStone_AstrallRecall_GetCooldown();
			local theCastStone = 0;
			
			-- Cast our spell first?
			if(theSpellCooldown == 0 and theSpellID ~= nil and UnitMana("player") > TITAN_Hearth_AstralRecallMana) then
			
				-- do it!
				CastSpell(theSpellID, "spell");
				
				-- got it!
				if(IsCurrentCast(theSpellID, "spell")) then
					
					-- don't cast our stone
					theCastStone = nil;
				end
			end
			
			-- cast our stone?
			if(theCastStone ~= nil) then
				-- Get the bag/slot.
				local theBag, theSlot = TITAN_HearthStone_GetBagSlotOfStone();
				
				-- Did we find it?
				if(theBag ~= TITAN_NotFoundBag) then
					-- Cast the object if we can.
					UseContainerItem(theBag, theSlot);
				end
			end
		end
	end	
end

-- ************************************* Non-Event Functions ************************************* --


-- ******************************************************************************************* --
-- Name: TitanPanelHearthStoneButton_BuildStringForHearthStone
-- Desc: WORKHORSE! This is the meat, where we get the heartstone out of our backpack, build
-- it's string, and use it.
-- ******************************************************************************************* --
function TitanPanelHearthStoneButton_BuildStringForHearthStone()

	local button, id 				= TitanUtils_GetButton(id, true);
	local theHearthStoneCooldown 	= 0;
	local theBag					= 0;
	local theSlot					= 0;
	local theSpellCooldown			= nil;
	local theSpellID				= nil;
	
	-- Get the cooldowns (stone / bag ) --
	theHearthStoneCooldown, theBag, theSlot = TITAN_HearthStone_GetCooldown();	
	theSpellCooldown, theSpellID = TITAN_HearthStone_AstrallRecall_GetCooldown();
	
	-- Turn it into a string --
	local theCooldownText = TITAN_HearthStone_GetFormattedTime(theHearthStoneCooldown);
	local theCooldownRichText;
				
	-- ************************ Hearthstone text ************************ --
	if(theHearthStoneCooldown ~= TITAN_HearthStone_NotFound) then

		-- if we have a time, use it	
		if(theHearthStoneCooldown ~= 0) then
			-- Got a time, record it.
			theCooldownRichText = TitanUtils_GetHighlightText(theCooldownText);
			
			-- Last check
			if(theCooldownText == "0" or theCooldownText == "1") then
				theCooldownRichText = TitanUtils_GetGreenText(TITAN_Hearthstone_Ready);
			end
		else
			-- Ready for use.
			theCooldownRichText = TitanUtils_GetGreenText(TITAN_Hearthstone_Ready);			
		end
	else
		-- Not found
		theCooldownRichText = TitanUtils_GetGreenText(TITAN_Hearthstone_NotFound);
	end	
	
	-- ************************ Spell text ************************ --
	if(theSpellCooldown ~= TITAN_HearthStone_NotFound) then
		
		-- colored.
		local theSpellRichCooldown;
		
		-- Ready for use?
		if(theSpellCooldown ~= 0) then
		
			-- We have it.. if its zero, its ready, otherwise, make the time.
			local theSpellCooldownText = TITAN_HearthStone_GetFormattedTime(theSpellCooldown);
			
			-- Last check
			if(theSpellCooldownText == "0" or theSpellCooldownText == "1") then
				-- ensure enough mana
				if (UnitMana("player") > TITAN_Hearth_AstralRecallMana) then
					-- Ready for use.
					theSpellRichCooldown = TitanUtils_GetGreenText(TITAN_Hearthstone_Ready);		
				else
					-- Ready for use.
					theSpellRichCooldown = TitanUtils_GetRedText(TITAN_Hearthstone_Ready);				
				end
			else	
				-- Got a time, record it.
				theSpellRichCooldown = TitanUtils_GetHighlightText(theSpellCooldownText);
			end
		else
		
			-- ensure enough mana
			if (UnitMana("player") > TITAN_Hearth_AstralRecallMana) then
				-- Ready for use.
				theSpellRichCooldown = TitanUtils_GetGreenText(TITAN_Hearthstone_Ready);		
			else
				-- Ready for use.
				theSpellRichCooldown = TitanUtils_GetRedText(TITAN_Hearthstone_Ready);				
			end
		end	
		
		-- Labels?
		if (TitanGetVar(TITAN_HearthStone_ID, "ShowLabelText")) then
			-- Copy it.
			theCooldownRichText = theCooldownRichText.." / "..TITAN_Hearthstone_AstralRecallLabel..theSpellRichCooldown;
		else
			-- Copy it.
			theCooldownRichText = theCooldownRichText.." / "..theSpellRichCooldown;	
		end
	end	
	
	-- return our results
	return TITAN_HearthStone_ID_LABEL, theCooldownRichText, theBag, theSlot;
end



-- ******************************************************************************************* --
-- Name: TitanPanelHearthStoneButton_GetButtonText
-- Desc: Gets our button text.. the text that appears on the actual bar, all the time.
-- ******************************************************************************************* --
function TitanPanelHearthStoneButton_GetButtonText(id)

	-- We do all the work in our tooltip, so no need to duplicate code, just call that.
	local theReturnFirst, theString, theBag, theSlot = TitanPanelHearthStoneButton_BuildStringForHearthStone();

	-- ******************************************** CAUTION ********************************************
	-- Don't ask me why, but this is backwards from what it should be.. I have no idea why this works flipped.	
	-- I must be missing something here.. but it "works".. its just backwards logic. Next version, hopefully
	-- with some community help, I'll have this resolved. I mentioned before, this is day 1 with lua, so
	-- bound to be some mistakes in here.
	if (not TitanGetVar(TITAN_HearthStone_ID, "ShowHearthLocation")) then
	
		-- Get the location
		local theOtherString 		= " - "..TITAN_HearthStone_ExtractHomeLocationString(theBag, theSlot);
		local theRichLocationString = TitanHearthStoneUtils_GetGoldText(theOtherString);
	
		-- Add these together
		theString = theString..theRichLocationString;
	end
	
	return theReturnFirst, theString;
	
end


-- ******************************************************************************************* --
-- Name: TitanPanelHearthStoneButton_GetTooltipText
-- Desc: Gets our tool-tip text, what appears when we hover over our item on the Titan bar.
-- ******************************************************************************************* --
function TitanPanelHearthStoneButton_GetTooltipText()
	
	-- We do all the work in our tooltip, so no need to duplicate code, just call that.
	local theReturnFirst, theString, theBag, theSlot = TitanPanelHearthStoneButton_BuildStringForHearthStone();
	local theOtherString = TITAN_HearthStone_ExtractHomeLocationString(theBag, theSlot);
	
	-- Add these together
	theString = theReturnFirst..theString..TitanHearthStoneUtils_GetGoldText(" - "..theOtherString);
	
	-- Hint
	if (not TitanGetVar(TITAN_HearthStone_ID, "ActivateOnClick")) then
		theString = theString.."\n"..TitanUtils_GetGreenText(TITAN_Hearthstone_Tooltip_Hints);
	end
	
	-- Done!
	return theString;

end


-- ******************************************************************************************* --
-- Name: TITAN_HearthStone_GetBagSlotOfStone
-- Desc: Returns the bag/slot (or the unknowns) of the stone.
-- ******************************************************************************************* --
function TITAN_HearthStone_GetBagSlotOfStone()

	-- our variables, default to not found.
	local theBag 	= TITAN_NotFoundBag;
	local theSlot 	= TITAN_NotFoundSlot;
	local theString = nil;
	
	-- Check each bag, looking for our hearthstone --
	for bag = 0, NUM_BAG_FRAMES do
	
		-- Loop over all the slots
    	for slot = 1, GetContainerNumSlots(bag) do
    	
    		-- Get the string name
    		local itemName =  TITAN_HearthStone_GetItemName(bag, slot);
    		
    		-- Is it right?
    		if (itemName == "Hearthstone") then
    		
    			-- found them.. set the results.
    			theBag 	= bag;
    			theSlot = slot;
    			
				
				-- break out!				
        		slot 	= GetContainerNumSlots(bag);
         		bag 	= NUM_BAG_FRAMES;
         		break;
    		end
    	end
    end
    
    return theBag, theSlot;
    
end

-- ******************************************************************************************* --
-- Name: TITAN_HearthStone_ExtractHomeLocationString
-- Desc: Extracts the home location of our hearthstone.
-- ******************************************************************************************* --
function TITAN_HearthStone_ExtractHomeLocationString(inBag, inSlot)

	-- Result
	local theResult = "";

	-- UPDATE: I wish there was a place to see all these functions. After ver 0.1 release, I found
	-- there was a function, GetBindLocation(), which basically does what you see below. So leaving the code in place
	-- I ever want to refer to it again.
	theResult = GetBindLocation();
	
	-- NOTE: Tooltips were maddening.. Its my first day with Lua, so I'm sure there
	-- is a better way to do this. For now, however, I'm simply going to build the
	-- tooltip when you load up the addon, then cache its location. When the timer comes around
	-- to refresh our location of the hearthstone, I'll see if any tooltip is up. If they are, I'll
	-- use the cached version, otherwise I refresh the cache.

--[[ *************************************************************************************************** 
--	 COMMENTING THIS ENTIRE SECTION OUT	
--   *************************************************************************************************** 

	-- only do this for our first run, or if they have no tooltip up.
	if(TITAN_CachedHome == nil or not GameTooltip:IsVisible()) then
		
		-- Setup the tooltip.
		GameTooltip:SetOwner(this);
		GameTooltip:Hide();
		local hasCooldown, repairCost = GameTooltip:SetBagItem(inBag, inSlot);
		
		-- Loop over the text description, find what we want, an run with it!
		for i = 0, GameTooltip:NumLines() do
		
			-- Get the tooltip lines.
			local theToolTipText = getglobal("GameTooltipTextLeft" .. i);
		   
			-- If its a valid tooltip...
			if(theToolTipText ~= nil) then
			
				-- Get its text
		   		local theText = theToolTipText:GetText();
		   		
		   		-- if it has text, then we can parse the line.
		   		if(theText ~= nil) then
	
					-- Get the string index of the first part.	   		
		   			local theStringIndex = string.find(theText, TITAN_HearthStonr_ParseReturnToLeft);
		   			
		   			-- Get the ending string index.
		   			local theEndingStringIndex = string.find(theText, TITAN_HearthStonr_ParseReturnToRight);
		   			
		   			-- if both are valid, use them
		   			if(theStringIndex ~= nil and theEndingStringIndex ~= nil) then
		   			
		   				-- Add to our string index, the length of that string.
		   				theStringIndex = strlen(TITAN_HearthStonr_ParseReturnToLeft) + theStringIndex;			   				
		   				
		   				-- Build our string.
		   				theResult = strsub(theText, theStringIndex, theEndingStringIndex - 2);
		   				
		   				-- copy to our cache
		   				TITAN_CachedHome = theResult;
		  				
		   				-- Debug statement
		   				--DEFAULT_CHAT_FRAME:AddMessage(string.format("Tooltip is(%d) stringindex(%d): %s", i, theStringIndex, theResult));
		   				
		   				-- Break out..
		   				break;
		   			end
		   		end
		   end
		end
	else
		-- use cached version.
		theResult = TITAN_CachedHome;
		
		--DEFAULT_CHAT_FRAME:AddMessage("CACHED: %s", TITAN_CachedHome);
	end


-- ********************************************* END COMMENTED OUT SECTION ********************************************* ]]--

	return theResult;
end


-- ******************************************************************************************* --
-- Name: TITAN_HearthStone_GetCooldown
-- Desc: This function looks through our bag, finds the hearthstone, returns its cooldown 
-- ******************************************************************************************* --
function TITAN_HearthStone_GetCooldown()

	local theCoolDown = TITAN_HearthStone_NotFound;
	local theString = nil;
	local theBag, theSlot = TITAN_HearthStone_GetBagSlotOfStone();
	
	-- Check each bag, looking for our hearthstone --
	if(theBag ~= TITAN_NotFoundBag) then
    		
		-- Lookup the itesm cooldown
 		local start, duration, enable = GetContainerItemCooldown(theBag, theSlot);
 		
 		-- Get the world real-time
 		local theRealTime = GetTime();
 		
 		-- set it to zero, meaning ready.
 		theCoolDown = 0;
 		
 		-- cooldown finished?
 		if(start ~= 0) then
 		
 			-- make sure its valid.
 			if(theRealTime > start) then
 			
 				-- Any time left?
 				if((theRealTime - duration) < start) then
 				
 					-- Compute that time
 					theCoolDown = (start - (theRealTime - duration));
 					
 					-- Debugging
 					--local theStringTime = TITAN_HearthStone_GetFormattedTime(theCoolDown);
 					--DEFAULT_CHAT_FRAME:AddMessage(string.format("GOT IT: %s", theStringTime));
 				end
 			end
 		end
 	end
      
    -- Done!
    return theCoolDown, theBag, theSlot;
end

-- ******************************************************************************************* --
-- Name: TITAN_Hearthstone_GetMatchingSpellId
-- Desc: Extracts the spell id from name
-- ******************************************************************************************* --
function TITAN_Hearthstone_GetMatchingSpellId (inName, inSpellbook, inStartID)

	local theResult = -1;
	
	-- adjust values
	if ( not inSpellbook ) then 
		inSpellbook = BOOKTYPE_SPELL; 
	end
	
	-- start ID
	if ( not inStartID ) then 
		inStartID = 1; 
	end
	
	
	-- find it.
	local theLoop = inStartID;
	local theName, theRankName;
	
	-- look it up.
	theName, theRankName = GetSpellName(theLoop, inSpellbook);
	while theName do
	
		-- find it?
		if ( theName == inName) then
			theResult = theLoop;
			break;
		end
		
		-- increment
		theLoop = theLoop + 1;
		theName, theRankName = GetSpellName(theLoop, inSpellbook)
	end
	
	return theResult;
end
	
-- ******************************************************************************************* --
-- Name: TITAN_HearthStone_AstrallRecall_GetCooldown
-- Desc: This function grabs the spell, finds the hearthstone, returns its cooldown 
-- ******************************************************************************************* --
function TITAN_HearthStone_AstrallRecall_GetCooldown()

	local theCoolDown = TITAN_HearthStone_NotFound;
	local theString = nil;
	local theSpellID = nil;
	
	-- optimization
	local thePlayerClass, theEnglishClass = UnitClass("player");
	if(thePlayerClass == TITAN_Hearthstone_Shaman) then

		-- lookup the spell ID by name
		theSpellID = TITAN_Hearthstone_GetMatchingSpellId(TITAN_Hearthstone_AstralRecall, BOOKTYPE_SPELL, 1);
		
		-- Check each bag, looking for our hearthstone --
		if(theSpellID ~= nil and theSpellID ~= -1) then
		
			-- Get the ti    		
			local theStartTime, theDuration = GetSpellCooldown(theSpellID, BOOKTYPE_SPELL);
			
			--DEFAULT_CHAT_FRAME:AddMessage(string.format("Time is: %d %d - %d", theStartTime, theDuration, (theStartTime - (GetTime() - theDuration))));
			
			-- Clear the cooldown (0 it)		
			theCoolDown = 0;
					
			-- Get the world real-time
	 		local theRealTime = GetTime();
	 		
	 		-- cooldown finished?
	 		if(theStartTime ~= 0) then
	 		
	 			-- make sure its valid.
	 			if(theRealTime > theStartTime) then
	 			
	 				-- Any time left?
	 				if((theRealTime - theDuration) < theStartTime) then
	 				
	 					-- Compute that time
	 					theCoolDown = (theStartTime - (theRealTime - theDuration));
	 					
	 					-- Debugging
	 					--local theStringTime = TITAN_HearthStone_GetFormattedTime(theCoolDown);
	 					--DEFAULT_CHAT_FRAME:AddMessage(string.format("GOT IT: %s", theStringTime));
	 				end
	 			end
	 		end
		end
	end		
      
    return theCoolDown, theSpellID;
end

-- ******************************************************************************************* --
-- Name: TITAN_HearthStone_GetFormattedTime
-- Desc: Turns time into a string. 
-- ******************************************************************************************* --
function TITAN_HearthStone_GetFormattedTime(time)

	-- adjusted time.
	local theTime = math.floor(time);
	
	-- result string.
	local theResult = "";
	
	-- Is it hours?
	if ( theTime > TITAN_HearthStone_Hours ) then

		-- sprintf!
		theResult = format("%s h", math.floor((theTime / 3600) + 0.5));
		
	elseif ( theTime > TITAN_HearthStone_Seconds ) then

		-- Minutes computation
		theResult = format("%s mins", math.ceil( ( theTime  / 60 )));
		
	else
	
		-- Its seconds, just use it.
		theResult = format("%s", theTime);
		
	end
	
	-- DONE!
	return theResult;
end	


-- ******************************************************************************************* --
-- Name: TITAN_HearthStone_GetItemName
-- Desc: Thanks to Capnbry for this one. 
-- ******************************************************************************************* --
function TITAN_HearthStone_GetItemName(bag, slot)

	local linktext = nil;
  
	if (bag == -1) then
		linktext = GetInventoryItemLink("player", slot);
	else
		linktext = GetContainerItemLink(bag, slot);
	end
	
	if linktext then
	
		local _,_,name = string.find(linktext, "^.*%[(.*)%].*$");
		
		return name;
	else
		return "";
  	end
  	
end


-- ******************************************************************************************* --
-- Name: TITAN_HearthStone_GetItemText
-- Desc: A bastard child of TITAN_HearthStone_GetItemName
-- ******************************************************************************************* --
function TITAN_HearthStone_GetItemText(bag, slot)

	local linktext = nil;
  
	if (bag == -1) then
		linktext = GetInventoryItemLink("player", slot);
	else
		linktext = GetContainerItemLink(bag, slot);
	end
	
	return linktext;
  	
end

-- ************************************* Titan Creates ************************************* --

-- ******************************************************************************************* --
-- Name: TitanPanelRightClickMenu_PrepareHearthStoneMenu
-- Desc: Preps our menu, if we have one. 
-- ******************************************************************************************* --
function TitanPanelRightClickMenu_PrepareHearthStoneMenu()

	-- Makes our mini-menu.
	TitanPanelRightClickMenu_AddTitle(TitanPlugins[TITAN_HearthStone_ID].menuText);
	TitanPanelRightClickMenu_AddToggleLabelText(TITAN_HearthStone_ID);
	TitanPanelRightClickMenu_AddCommand(TITAN_PANEL_MENU_HIDE, TITAN_HearthStone_ID, TITAN_PANEL_MENU_FUNC_HIDE);

	-- Show location
	info = {};
	info.text = TITAN_HearthStone_MENU_SHOW_LOCATION;
	info.func = TitanPanelHearthStoneButton_ToggleShowLocation;
	info.checked = TitanUtils_Toggle(TitanGetVar(TITAN_HearthStone_ID, "ShowHearthLocation"));
	UIDropDownMenu_AddButton(info);
	
	-- Cast on click
	info = {};
	info.text = TITAN_HearthStone_CastOnClick;
	info.func = TitanPanelHearthStoneButton_ToggleCastOnClick;
	info.checked = TitanUtils_Toggle(TitanGetVar(TITAN_HearthStone_ID, "ActivateOnClick"));
	UIDropDownMenu_AddButton(info);
	
	-- info about plugin
	local info = {};
	info.text = TITAN_HearthStone_ABOUT_TEXT;
	info.value = "DisplayAbout";
	info.hasArrow = 1;
	UIDropDownMenu_AddButton(info);
end


-- ******************************************************************************************* --
-- Name: TitanPanelHearthStoneButton_ToggleShowLocation
-- Desc: Callback, toggles the show location option
-- ******************************************************************************************* --
function TitanPanelHearthStoneButton_ToggleShowLocation()

	-- BELOW, old version before I realized about TITAN_NIL
--	local theFlag = TitanGetVar(TITAN_HearthStone_ID, "ShowHearthLocation");
--	if(theFlag ~= nil) then
--		TitanSetVar(TITAN_HearthStone_ID, "ShowHearthLocation", nil);
--	else
--		TitanSetVar(TITAN_HearthStone_ID, "ShowHearthLocation", 1);
--	end
	
	TitanToggleVar(TITAN_HearthStone_ID, "ShowHearthLocation");
	TitanPanelButton_UpdateButton(TITAN_HearthStone_ID);
end

-- ******************************************************************************************* --
-- Name: TitanPanelHearthStoneButton_ToggleCastOnClick
-- Desc: Callback, toggles the cast on click option
-- ******************************************************************************************* --
function TitanPanelHearthStoneButton_ToggleCastOnClick()

-- BELOW old version before I realized about TITAN_NIL
--	local theFlag = TitanGetVar(TITAN_HearthStone_ID, "ActivateOnClick");
--	if(theFlag ~= nil) then
--		TitanSetVar(TITAN_HearthStone_ID, "ActivateOnClick", nil);
--	else
--		TitanSetVar(TITAN_HearthStone_ID, "ActivateOnClick", 1);
--	end

	TitanToggleVar(TITAN_HearthStone_ID, "ActivateOnClick");
	TitanPanelButton_UpdateButton(TITAN_HearthStone_ID);
end


-- ******************************************************************************************* --
-- Name: TitanPanelHearthStoneRightClickMenu_OnLoad
-- Desc: Preps our menu, I think.. copied this from other titan add-ons.
-- ******************************************************************************************* --
function TitanPanelHearthStoneRightClickMenu_OnLoad()
	UIDropDownMenu_Initialize(TitanPanelHearthStone_Button_Menu, TitanPanelHearthStoneRightClickMenu_CreateMenu, "MENU");
end


-- ******************************************************************************************* --
-- Name: TitanPanelHearthStoneRightClickMenu_CreateMenu
-- Desc: Preps our right-click menu.
-- ******************************************************************************************* --
function TitanPanelHearthStoneRightClickMenu_CreateMenu(level)

	local info = {};
	
	if ( level == 2 ) then		
		if ( UIDROPDOWNMENU_MENU_VALUE == "DisplayAbout" ) then
			info.text = TITAN_HearthStone_ABOUT_POPUP_TEXT;
			info.value = "AboutTextPopUP";
			info.notClickable = 1;
			info.isTitle = 0;
			UIDropDownMenu_AddButton(info, level);
		end
	else
		TitanPanelRightClickMenu_PrepareHearthStoneMenu();
	end
end


-- ******************************************************************************************* --
-- Name: TitanHearthStoneUtils_GetGoldText
-- Desc: turns text gold
-- ******************************************************************************************* --
function TitanHearthStoneUtils_GetGoldText(inText)
	if (inText) then
		return TITAN_HearthStone_COLOR_GOLD..inText..FONT_COLOR_CODE_CLOSE;
	end
end